#-*- coding: utf-8 -*-

import sqlite3
import random
import math

from django.shortcuts import render, redirect
from grille_de_l_amour.appli.models import Pixel, NombreImages
from ajout import ajout


# Respecte l'orde des couleurs telles qu'affichées à l'écran dans la palette
TAB_COULEUR = ['000000', 'ffffff', 'ff0000', '0000ff', '00ff00', 'ffff00', 'ff00ff', '00ffff' ]


#########################################
# Partie gérant l'affichage de la carte #
#########################################


def deg2num(lat_deg, lon_deg, zoom=16):
    """
        Formule qui permet d'obtenir le nom y/x de la tuile en fonction de ses coordonnées
        Récupérée ici : https://wiki.openstreetmap.org/wiki/Slippy_map_tilenames
    """

    lat_rad = math.radians(lat_deg)
    n = 2.0 ** zoom
    xtile = int((lon_deg + 180.0) / 360.0 * n)
    ytile = int((1.0 - math.log(math.tan(lat_rad) + (1 / math.cos(lat_rad))) / math.pi) / 2.0 * n)

    return (xtile, ytile)

def origine(request):
    """
        url proposée par défaut
    """

    # Orléans
    return redirect (page_carte, niv_zoom=14, latitude=47.9032, longitude=1.9114)

    # Nantes
    #return redirect (page_carte, niv_zoom=14, latitude=47.2138, longitude=-1.5562)


def page_carte(request, niv_zoom, latitude, longitude):
    """
        construit la portion de html constituée des <img> relatives aux tuiles
        et l'envoie au template de la carte (qui ne l'échappera pas)
    """

    # Correspondance entre les couleurs les les skins de CloudMade 
    # disponibles ici: http://maps.cloudmade.com/editor
    # Respecte l'orde des couleurs telles qu'affichées à l'écran dans la palette

    TAB_PALETTE = [
                '16340',        # noir
                '45499',        # blanc
                '33602',        # rouge 
                '36041',        # bleu
                '7543',         # vert 
                '39461',        # jaune
                '7134',         # violet
                '20538'         # turquoise
                ]

    LETTRE = ['a', 'b', 'c']

    #  ajustement nécéssaire entre le passage de 256*256 à 64*64
    zoom = int(niv_zoom) + 2 
    coord = deg2num(float(latitude), float(longitude), zoom)

    # pour recentrer
    y = coord[1] - 4
 
    top = 0
    cpt = 0
    grille = ''

    for i in range(10):

        # pour recentrer
        x = coord[0] - 6
 
        left = 0

        for j in range(13):

            for pixel in Pixel.objects.filter(nb = cpt):

                couleur_pixel = int(pixel.couleur)

            ligne = '<img class="tuile" style="left: {0}px; top: {1}px;" '.format(left, top)
            ligne += 'src="http://{0}.tile.cloudmade.com/75b297222833420cb33b584f6f2d24c8/'.format(LETTRE[random.randint(0, 2)])
            ligne += '{0}/64/{1}/{2}/{3}.png">\n'.format(TAB_PALETTE[couleur_pixel], zoom, x, y)

            grille += ligne
            x += 1
            left += 64
            cpt += 1

        y += 1
        top += 64

    return render(request, 'tpl_carte.html', {'liste_img': grille})


#######################################
# Partie gérant l'interface de dessin #
#######################################


def page_grille(request, selcoul=0):

    cpt = 0
    grille = ''
    nb_total = NombreImages.objects.get(nom='total').valeur

    for i in range(10):

        grille += """               <tr>"""

        for j in range(13):

            for pixel in Pixel.objects.filter(nb = cpt):

                couleur_pixel = TAB_COULEUR[int(pixel.couleur)]
                grille += '<td style="background:#{0}"></td>'.format(couleur_pixel)

            cpt += 1

        grille += '</tr>\n'

    return render(request, 'tpl_grille.html', {'tab_grille': grille, 'selcoul': selcoul, 'nb_total': nb_total})


def modif_pixel(request, couleur, num_case):

    selcoul = int(couleur)
    nb_case = int(num_case)

    if ((nb_case > 129) or (selcoul > 7)):

        return redirect('accueil_grille')

    for pixel in Pixel.objects.filter(nb = int(num_case)):
        pixel.couleur = selcoul
        pixel.save()

    return page_grille (request, selcoul)


def ajout_image(request, couleur):

    nb = ajout()

    return page_grille (request, couleur)


###########################################
# Partie gérant l'affichade de la matrice #
###########################################


def page_matrice(request):
    """
        Construit la liste d'images à afficher en dessous de matrice.png
        quand la dernière ligne n'est pas complète.

    """

    liste_img = ''
    nb_total = int(NombreImages.objects.get(nom='total').valeur)
    nb_lignes = nb_total/30
    nb_images_rest = nb_total%30

    left = 0
    top = nb_lignes*10

    name = 8999999999 - nb_total + nb_images_rest

    for i in range(nb_images_rest): 

            ligne = '''<img class="vignette" style="left: {0}px; top: {1}px;" src="/media/images/grille-{2}.png">\n'''.format(left, top, name)
            liste_img += ligne

            left += 13 # décale de la largeur d'une image
            name -= 1

    return render(request, 'tpl_matrice.html', {'liste_img': liste_img})


#########################################
# Partie gérant l'affichade de la ligne #
#########################################


def page_ligne(request):

    
    liste_img = ''
    left = 0
    nb_total = int(NombreImages.objects.get(nom='total').valeur)
    first_name = 9000000000 - nb_total
    name = first_name

    for i in range(1000): # à remplacer par nb_total pour avoir la colonne complète

            ligne = '''<img class="image390" style="left: {0}px; top: 0px;" src="/media/images/grille-{1}.png">\n'''.format(left, name)
            liste_img += ligne

            left += 390
            name += 1

    return render(request, 'tpl_ligne.html', {'liste_img': liste_img})


###########################################
# Partie gérant l'affichade de la colonne #
###########################################


def page_colonne(request):

    liste_img = ''
    top = 0
    nb_total = int(NombreImages.objects.get(nom='total').valeur)
    first_name = 9000000000 - nb_total
    name = first_name

    for i in range(1000): # à remplacer par nb_total pour avoir la colonne complète

            colonne = '''<img class="image390" style="left: 0px; top: {0}px;" src="/media/images/grille-{1}.png">\n'''.format(top, name)
            liste_img += colonne

            top += 300
            name += 1

    return render(request, 'tpl_colonne.html', {'liste_img': liste_img})


####################################################
# Partie gérant la création et l'affichage du film #
####################################################


def page_film(request):

    nom_film_temp = random.randint(0, 10000)

    #os.chdir("./grille_de_l_amour/static")
    #os.system('''mencoder "mf://grille_de_l_amour/static/images/*.png" -fps 7 -o ./grille_de_l_amour/static/film-{0}.avi -ovc lavc -lavcopts vcodec=mjpeg'''.format(nom_film_temp))
    #os.system('mencoder -oac copy -ovc copy -idx -o film.avi film-{0}.avi film-old.avi'.format(nom_film_temp))
    #os.system('ffmpeg -i film.avi -b 1024000 -f flv -y film.flv')
    #os.system('rm film-{0}.avi'.format(nom_film_temp))

    return render(request, 'tpl_film.html')


########################
# Partie gérant le 404 #
########################


def accueil(request):

    return redirect('accueil_grille')


